# helppanel.tcl --
#
#       Snit widget helppanel
#
# Copyright (c) 2003    Dr. Detlef Groth <dgroth@gmx.de>
#
# See the file "license.terms" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
# 
# RCS: @(#) $Id: helppanel.tcl,v 0.1 2003/11/30 18:26:59 detlef_groth Exp $
# Last Modified : <040120.1155>
lappend auto_path [file join [file dirname [info script]] ..]
package require snit 0.91
package require BWidget
package require dgwidgets

snit::widget SnitTree {
    expose tree
    option -opennodes [list]
    option -browsecmd
    delegate option * to tree
    delegate method * to tree
    variable found
    constructor {args} {
        set sw [ScrolledWindow $win.f -relief sunken -borderwidth 2]
        pack $sw -fill both -expand yes
        install tree using Tree $win.tree -showlines false
        $sw setwidget $win.tree
        #pack $tree -side left -fill both -expand true
        $tree configure -opencmd [mymethod _TreeOpenClose 1]
        $tree configure -closecmd [mymethod _TreeOpenClose 0]
        $tree bindText <1> [mymethod _TreeSelect]
        $tree configure -background white
        $self configurelist $args
        if {[info exists options(-browsecmd)]} {
            $tree bindText <Double-1> [mymethod _BrowseCmd]
        }
    }
    method autoset {{root root}} {
        foreach node [$tree nodes $root] {
            if {$root ne "root"} {
                if {[$tree itemcget $root -open]} {
                    $tree itemconfigure $root -image appbookopen16
                } else {
                    $tree itemconfigure $root -image appbook16
                }
            }
            $self autoset $node
        }
    }
    onconfigure -opennodes {values} {
        set options(-opennodes) [list]
        foreach node [luniq $values] {
            if {[$tree exists $node]} {
                lappend $options(-opennodes) $node
            } else {
                set options(-opennodes) [ldelete $options(-opennodes) $node]
            }
            #$self _TreeOpenClose 1 $node 
            $tree opentree $node false
        }
        
    }
    method itemcgetUp {node opt {val ""}} {
        # gets all options walking the tree up to the root
        # returns a list
        set parent [$self parent $node]
        lappend val [$self itemcget $node $opt]
        if {$parent eq "root"} { return $val }
        $self itemcgetUp $parent $opt $val
        
    }
    method itemcgetDown {node opt value} {
     
    }
    method itemSearch {startnode opt value} {
        #puts stderr ":$startnode $value"
        if {$startnode ne "root"} {
            #puts stderr ":[$self itemcget $startnode $opt] = $value"
            if {[string first $value [$self itemcget $startnode $opt]] >= 0} {
                lappend found $startnode
            }
        } else {
            set found [list]
        }
        #puts stderr ":1$startnode $value"
        foreach node [$self nodes $startnode] {
            #puts stderr ":2 $node $startnode $value"
            $self itemSearch $node $opt $value
        }
        return $found
    }
    method itemShow {node} {
        # makes an unvisible node visible
        set parent [$self parent $node]
        if {$parent eq "root"} { return }
        $self opentree $parent false
        $self see $node
    }
    method _TreeOpenClose {idx node} {
        if {$idx == 1} {
            lappend options(-opennodes) $node
        } else {
            #            puts "try   deleting $node: $options(-opennodes)"
            set options(-opennodes) [ldelete $options(-opennodes) $node]
            #            puts "after deleting $node: $options(-opennodes)"
        }
        $tree itemconfigure $node -image [lindex {appbook16 appbookopen16} $idx]
    }
    method _BrowseCmd {args} {
        $options(-browsecmd) $tree [$tree selection get]
    }
    method _TreeSelect {item} {
        $tree selection set $item
    }
    method _BrowseCmd {args} {
        $options(-browsecmd) $tree [$tree selection get]
    }
    proc luniq {list} {
	foreach val $list {
            set a($val) 1
	}
	return [array names a]
    }
    proc ldelete { list value } {
        set ix [lsearch -exact $list $value]
        if {$ix >= 0} {
            #            puts "found $ix"
            return [lreplace $list $ix $ix]
        } else {
            return $list
        }
    }
}
snit::widget HelpPanel {
    expose tree
    expose notebook
    expose combo
    expose listbox
    expose bookmarks
    delegate option -labeltext to mcombo
    delegate option -textvariable to combo
    delegate option -width to notebook
  
    option -browsecmd
    option -nbraisecmd
    delegate method addHistory to mcombo
    variable searchlistbox
    variable mcombo
    constructor {args} {
        install notebook using NoteBook $win.nb -side top -width 200 -height 150
        pack $notebook -fill both -expand yes -side top
        # the contents panel
        $win.nb insert end tree -text " Contents "
        set f [$win.nb getframe tree]
        install tree using SnitTree $f.tree -showlines false
        pack $tree -side left -fill both -expand true
      
        $tree bindText <Enter> [mymethod _Hilight $tree 1]
        $tree bindText <Leave> [mymethod _Hilight $tree 0]
        
        # the index panel
        $win.nb insert end index -text " Index "
        set f [$win.nb getframe index]
     
        pack [SearchListBox $f.slb -labeltext "Search: "] -side left -fill both -expand true
        set searchlistbox $f.slb
        set combo [$f.slb combo]
        set mcombo [$f.slb mcombo]
        set listbox [$f.slb listbox]
      
        $listbox bindText <Enter> [mymethod _Hilight $listbox 1]
        $listbox bindText <Leave> [mymethod _Hilight $listbox 0]
        
        # the bookmarks panel
        $win.nb insert end bookmarks -text " Bookmarks "
        set f [$win.nb getframe bookmarks]
        install bookmarks using BookMarkBox $f.bm -labeltext Thema
        pack $bookmarks -side left -fill both -expand true
        $bookmarks bindText <Enter> [mymethod _Hilight $bookmarks 1]
        $bookmarks bindText <Leave> [mymethod _Hilight $bookmarks 0]
        $self configurelist $args
        if {[info exists options(-browsecmd)]} {
            #            $tree bindText <Double-1> [mymethod _BrowseCmd]
            $tree configure -browsecmd  $options(-browsecmd)
            $searchlistbox configure -browsecmd  $options(-browsecmd)
            $bookmarks configure -browsecmd  $options(-browsecmd)
        } 
        if {[info exists options(-nbraisecmd)]} {
            $win.nb itemconfigure bookmarks -raisecmd $options(-nbraisecmd)
        }
       
    }
    method _Hilight {w on node} {
        if {$on} {
            $w itemconfigure $node -fill blue 
        } else {
            $w itemconfigure $node -fill black
        }
    }
}
snit::widget BookMarkBox {
    delegate option -labeltext to label as -text
    delegate option -textvariable to entry
    delegate option -urlvariable to url as -textvariable
    option -browsecmd
    delegate option * to listbox
    delegate method * to listbox
    variable entry
    variable label
    variable add
    variable del
    variable listbox
    constructor {args} {
        frame $win.f1
        set label [label $win.f1.lbl] 
        set entry [entry $win.f1.entry]
        frame $win.f1b
        set urllabel [label $win.f1b.lbl -text URL:] 
        set url [label $win.f1b.urllbl]
        frame $win.f2
        set add [button $win.f2.add -text " Add " -width 10 -command [mymethod Add]]
        set del [button $win.f2.del -text " Delete" -width 10 -command [mymethod Delete]] 
        frame $win.f3
        frame $win.f3.f
        ListBox $win.f3.f.lb -xscrollcommand [list $win.f3.xscroll set] \
                  -yscrollcommand [list $win.f3.f.yscroll set]
        set listbox $win.f3.f.lb
        pack $listbox -side left -fill both -expand true
        
        #set component(listbox) $win.f.lb 
        set platform [lindex $::tcl_platform(os) 0]
        #if {$platform eq "Windows"} {
           
        #}
        scrollbar $win.f3.xscroll -orient horizontal  -command [list $win.f3.f.lb xview]
        scrollbar $win.f3.f.yscroll -orient vertical  -command [list $win.f3.f.lb yview]
        pack $win.f3.f.yscroll -side right -fill y
        pack $win.f3.xscroll -side bottom -fill x
       
        $listbox bindText <1> [mymethod _ListBoxSelect]
       
        
        pack $win.f1 $win.f1b $win.f2 -side top -fill x -expand no -padx 5 -pady 5
        pack $win.f3 $win.f3.f  -side top -fill both -expand yes
        pack $label $urllabel  -side left -fill x -expand no -padx 5 -pady 5
        pack $entry $add $del $url -side left -fill x -expand yes -padx 5 -pady 5
        $self configurelist $args
        if {[info exists options(-browsecmd)]} {
            $listbox bindText <Double-1> [mymethod _BrowseCmd]
        }
        $listbox configure -background white
      
    }
    method insert {btext burl} {
        $listbox insert end #auto -text $btext -data $burl -image filedocument16
    }
    method getitems {} {
        set ret [list]
        foreach item [$listbox items 0 end] {
            lappend ret [$listbox itemcget $item -text]
            lappend ret [$listbox itemcget $item -data]
        }
        return $ret
    }
    method Add {} {
        $listbox insert end #auto -text "[$entry get]" -data "[$url cget -text]" -image filedocument16
    }
    method Delete {} {
        set nodes [$listbox selection get]
        if {[llength $nodes] > 0} {
            set answer [tk_messageBox -title "Question!" -message "Really delete [llength $nodes] items?" -type yesno -icon question]
            if { $answer } {
                $listbox delete $nodes
            } 
        }
    }
    method _BrowseCmd {node} {
        $options(-browsecmd) $listbox $node
    }
    method _ListBoxSelect {item} {
        $listbox selection set $item
    }
}

snit::widget SearchListBox {
    delegate option -labeltext to mcombo
    delegate option -textvariable to mcombo
    delegate method addHistory to mcombo
    delegate option * to listbox
    delegate method * to listbox
    option -browsecmd
    #variable component
    variable lasttext "nnnnn"
    expose mcombo
    expose combo
    expose listbox
    constructor {args} {
        install mcombo using mComboBox $win.combo
        pack $mcombo -side top -fill x -expand no
       
        set combo [$mcombo combo]
#        set component(entry) [$mcombo entry]
#        set component(label) [$mcombo label]
        pack [frame $win.f] -side top -fill both -expand true
        install listbox using ListBox $win.f.lb -xscrollcommand [list $win.xscroll set] \
              -yscrollcommand [list $win.f.yscroll set]
        pack $listbox -side left -fill both -expand true
        
        #set component(listbox) $win.f.lb 
        set platform [lindex $::tcl_platform(os) 0]
        if {$platform eq "Windows"} {
            $listbox configure -background white
        }
        scrollbar $win.xscroll -orient horizontal  -command [list $win.f.lb xview]
        scrollbar $win.f.yscroll -orient vertical  -command [list $win.f.lb yview]
        pack $win.f.yscroll -side right -fill y
        pack $win.xscroll -side bottom -fill x
        $combo bind <Key> [mymethod _Search %A %K]
        $listbox bindText <1> [mymethod _ListBoxSelect]
        $listbox configure -background white
        $self configurelist $args
        if {[info exists options(-browsecmd)]} {
           
            $listbox bindText <Double-1> [mymethod _BrowseCmd]
            $combo configure -command [mymethod _BrowseCmd]
            
        }
    }
    method _BrowseCmd {args} {
        if {[llength $args] > 0} {
            # listbox invoke
            #puts "listbox $args"
            $options(-browsecmd) $listbox [lindex $args 0]
        } else {
            #combo invoke
            #puts "combo [$component(listbox) selection get]"
            $options(-browsecmd) $listbox [$listbox selection get]
        }
            
        
    }
    method _ListBoxSelect {item} {
        #puts "item: $item"
        $listbox selection set $item
    }
    method _Search {key symbol} {
        #puts "$key $symbol"
        set stext [$combo cget -text]
        if {$lasttext eq $stext && $key eq "Enter"} {
            #puts Enter
            if {[info exists options(-browsecmd)]} {
                $options(-browsecmd) $listbox [$listbox selection get]
            }
            #CHM:Browse $listbox [$listbox selection get]
            $combo addHistory [$listbox itemcget [$listbox selection get] -text]
            return
        } elseif {$symbol eq "BackSpace"} {
            set stext [string range $stext 0 end-1]
        } else {
            set stext $stext$key
        }
        set x 0
        foreach item [$listbox items] {
            set text [$listbox itemcget $item -text]
            if  {[string match -nocase "$stext*" $text]} {
                $listbox selection set $item
                $listbox see [$listbox items end] ;# to put selected item on top
                $listbox see $item
           
                set lasttext $stext
                break
            } 
        }
    }
    method component {w args} {
        if {[llength $args] == 0} {
            # user want's to save component in one variable
            if {[info exists component($w)]} {
                return $component($w)
            } else {
                error "Component does not exist! Components are: [array names component]!"
            }
        } else {
            # user use it as a subcommand
            if {[info exists component($w)]} {
                
                if {[lindex $args 0] eq "bind"} {
                    eval { bind $component($w) } [lrange $args 1 end]
                } else {
                   eval { $component($w) } $args
                }
            } else {
                error "Component does not exist! Components are: [array names component]!"
            }
        }
    }
}
proc Test_HelpPanel {} {
    proc browse:test {args} {
        puts "browse test: $args"
        .child.s addHistory $args
    
    }
    toplevel .child
    pack [HelpPanel .child.s -labeltext "Search:" -browsecmd browse:test] -side top -expand yes -fill both
    .child.s addHistory "test"
    for {set i 0} {$i < 50} {incr i 1} {
        .child.s listbox insert end #auto -text "$i item" -image filedocument16
    }
    set lb [.child.s listbox]
    for {set i 0} {$i < 20} {incr i 1} {
        .child.s tree insert end root node$i -text "$i item" -image appbook16
        for {set j 0} {$j < 20} {incr j} {
            .child.s tree insert end node$i node${i}.$j -text "$i $j item" -image filedocument16
        }
    }
    .child.s notebook itemconfigure index -raisecmd {browse:test}
    .child.s notebook raise tree
}
#Test_HelpPanel
proc Test_Search_ListBox {} {
    proc browse:test {args} {
        puts "browse test: $args"
        .child.s addHistory $args
        
    }
    toplevel .child
    pack [SearchListBox .child.s -labeltext "Search:" -browsecmd browse:test] -side top -expand yes -fill both
    .child.s addHistory "test"
    for {set i 0} {$i < 50} {incr i 1} {
        .child.s insert end #auto -text "$i item"
    }
    set lb [.child.s component listbox]
    
    #.s component listbox bindText <Button-3> 
    #bind $lb <Button-3> {puts pressed3}
}
